@@ -1,8 +1,7 @@
-FULL_VERSION  = 27.1.0
-MAJOR_VERSION = 27
+FULL_VERSION  = 2.7.2.8
+MAJOR_VERSION = 2.7.2
 
 SHLIB      = libg++.so.$(FULL_VERSION)
 MSHLINK    = libg++.so.$(MAJOR_VERSION)
 BUILD_LIBS = $(ARLIB) $(SHLIB) $(SHLINK) $(MSHLINK)
 SHFLAGS    = -Wl,-soname,$(MSHLINK)
-SHCURSES   = #-lcurses
