@@ -368,7 +368,7 @@ struct	Record
 
 typedef struct Record 	RecordType;
 typedef RecordType *	RecordPtr;
-typedef int		boolean;
+typedef int		dhry_boolean;
 
 #define	TRUE		1
 #define	FALSE		0
@@ -391,7 +391,7 @@ void Proc2(OneToFifty	*IntParIO);
 void Proc3(RecordPtr	*PtrParOut);
 void Proc4();
 void Proc5();
-boolean Func3(Enumeration	EnumParIn);
+dhry_boolean Func3(Enumeration	EnumParIn);
 void Proc6(REG Enumeration	EnumParIn, REG Enumeration	*EnumParOut);
 void Proc7(OneToFifty IntParI1, OneToFifty IntParI2, OneToFifty *IntParOut);
 void Proc8(Array1Dim	Array1Par, 
@@ -399,8 +399,8 @@ void Proc8(Array1Dim	Array1Par,
       OneToFifty IntParI1, 
       OneToFifty IntParI2);
 Enumeration Func1(CapitalLetter	CharPar1, CapitalLetter	CharPar2);
-boolean Func2(String30	StrParI1, String30	StrParI2);
-boolean Func3(Enumeration	EnumParIn);
+dhry_boolean Func2(String30	StrParI1, String30	StrParI2);
+dhry_boolean Func3(Enumeration	EnumParIn);
 
 void mystrcpy(String30 s, char* t)
 {
@@ -426,7 +426,7 @@ main()
  * Package 1
  */
 Int		IntGlob;
-boolean		BoolGlob;
+dhry_boolean		BoolGlob;
 char		Char1Glob;
 char		Char2Glob;
 Array1Dim	Array1Glob;
@@ -590,7 +590,7 @@ void Proc3(RecordPtr	*PtrParOut)
 
 void Proc4()
 {
-	REG boolean	BoolLoc;
+	REG dhry_boolean	BoolLoc;
 
 	BoolLoc = Char1Glob == 'A';
 	BoolLoc |= BoolGlob;
@@ -663,7 +663,7 @@ Enumeration Func1(CapitalLetter	CharPar1, CapitalLetter	CharPar2)
 		return (Ident2);
 }
 
-boolean Func2(String30	StrParI1, String30	StrParI2)
+dhry_boolean Func2(String30	StrParI1, String30	StrParI2)
 {
 	REG OneToThirty		IntLoc;
 	REG CapitalLetter	CharLoc;
@@ -691,7 +691,7 @@ boolean Func2(String30	StrParI1, String30	StrParI2)
 	}
 }
 
-boolean Func3(Enumeration	EnumParIn)
+dhry_boolean Func3(Enumeration	EnumParIn)
 {
 	REG Enumeration	EnumLoc;
 
