@@ -67,13 +67,15 @@ DEFUN(_IO_file_doallocate, (fp),
   register char *p;
   struct stat st;
 
+#ifndef _IO_USE_LIBC
   /* If _IO_cleanup_registration_needed is non-zero, we should call the
      function it points to.  This is to make sure _IO_cleanup gets called
      on exit.  We call it from _IO_file_doallocate, since that is likely
      to get called by any program that does buffered I/O. */
   if (_IO_cleanup_registration_needed)
     (*_IO_cleanup_registration_needed)();
-  
+#endif
+
   if (fp->_fileno < 0 || _IO_SYSSTAT (fp, &st) < 0)
     {
       couldbetty = 0;
@@ -92,9 +94,7 @@ DEFUN(_IO_file_doallocate, (fp),
       size = _IO_BUFSIZ;
 #endif
     }
-  p = ALLOC_BUF(size);
-  if (p == NULL)
-    return EOF;
+  ALLOC_BUF(p, size, EOF);
   _IO_setb(fp, p, p+size, 1);
   if (couldbetty && isatty(fp->_fileno))
     fp->_flags |= _IO_LINE_BUF;
