@@ -42,3 +42,7 @@ DEFUN(_IO_fwrite, (buf, size, count, fp),
   else
     return written/size;
 }
+
+#if defined(__ELF__) && defined(_IO_USE_LIBC)
+#pragma weak fwrite = _IO_fwrite
+#endif
