@@ -35,11 +35,14 @@ the executable file might be covered by the GNU General Public License. */
 #include <string.h>
 #include <limits.h>
 #include "floatio.h"
-#ifdef __linux__
+#if defined(__linux__) && !defined(_IO_USE_DTOA)
 #include <printf.h>
 extern "C" int __printf_fp __P ((FILE *,
 				const struct printf_info *,
-				const void **const));
+				const void *const*));
+#else
+extern "C" int __cvt_double (double, int, int, int *, int,
+			     char *, char *);
 #endif
 
 #define	BUF		(MAXEXP+MAXFRACT+1)	/* + decimal point */
@@ -327,11 +330,15 @@ READ_INT(bool)
 istream& istream::operator>>(long double& x)
 {
     if (ipfx0())
-#ifdef __linux__
+      {
+#if defined(__linux__) && !defined(_IO_USE_DTOA)
 	scan("%Lg", &x);
 #else
-	scan("%lg", &x);
+	double y;
+	scan("%lg", &y);
+	x = y;
 #endif
+      }
     return *this;
 }
 
@@ -410,7 +417,7 @@ static void write_int(ostream& stream, unsigned LONGEST val, int sign)
 #define WRITE_BUF_SIZE (10 + sizeof(unsigned LONGEST) * 3)
     char buf[WRITE_BUF_SIZE];
     register char *buf_ptr = buf+WRITE_BUF_SIZE; // End of buf.
-    char *show_base = "";
+    const char *show_base = "";
     int show_base_len = 0;
     int show_pos = 0; // If 1, print a '+'.
 
@@ -426,7 +433,7 @@ static void write_int(ostream& stream, unsigned LONGEST val, int sign)
 	    *--buf_ptr = '0';
     }
     else if ((stream.flags() & ios::basefield) == ios::hex) { // Hex
-	char *xdigs = (stream.flags() & ios::uppercase) ? "0123456789ABCDEF0X"
+	const char *xdigs = (stream.flags() & ios::uppercase) ? "0123456789ABCDEF0X"
 	    : "0123456789abcdef0x";
 	do {
 	    *--buf_ptr = xdigs[val & 15];
@@ -562,6 +569,10 @@ ostream& ostream::operator<<(unsigned long long n)
 #endif /*__GNUC__*/
 
 #ifdef __linux__
+#ifdef _IO_USE_DTOA
+ostream& ostream::operator<<(long double n)
+{ return operator<<((double)n); }
+#else
 ostream& ostream::operator<<(long double n)
 {
     if (opfx()) {
@@ -580,21 +591,30 @@ ostream& ostream::operator<<(long double n)
 	// Do actual conversion.
 	struct printf_info info;
 
+#if 1
+	// Not all printf_info's have the extra field.
+	memset (&info, 0, sizeof (info));
+#else
+	info.is_short = 0;
+	info.is_long = 0;
+	info.space = 0;
+	info.group = 0;
+	info.extra = 0;
+#endif
 	info.prec = prec;
 	info.width = width (0);
 	info.spec = format_char;
 	info.is_long_double = 1;
-	info.is_short = 0;
-	info.is_long = 0;
-	info.alt = flags() & ios::showpoint;
-	info.space = 0;
-	info.left = flags() & ios::left;
-	info.showsign = flags() & ios::showpos;
-	info.group = 0;
+	info.alt = (flags() & ios::showpoint) != 0;
+	info.left = (flags() & ios::left) != 0;
+	info.showsign = (flags() & ios::showpos) != 0;
 	info.pad = fill();
 
+#if 0
 	_IO_va_list p = (_IO_va_list) &n;
-
+#else
+	const void *p = (const void *) &n;
+#endif
 	if (__printf_fp (rdbuf(), &info, &p) < 0)
 	    set(ios::badbit|ios::failbit); // ??
 	osfx();
@@ -602,6 +622,7 @@ ostream& ostream::operator<<(long double n)
     return *this;
 }
 #endif
+#endif
 
 ostream& ostream::operator<<(double n)
 {
@@ -622,23 +643,33 @@ ostream& ostream::operator<<(double n)
 	  prec = 6; /* default */
 
 	// Do actual conversion.
-#ifdef __linux__
+#if defined(__linux__) && !defined(_IO_USE_DTOA)
 	struct printf_info info;
 
-	info.prec = prec;
-	info.width = width (0);
-	info.spec = format_char;
+#if 1
+	// Not all printf_info's have the extra field.
+	memset (&info, 0, sizeof (info));
+#else
 	info.is_long_double = 0;
 	info.is_short = 0;
 	info.is_long = 0;
-	info.alt = flags() & ios::showpoint;
 	info.space = 0;
-	info.left = flags() & ios::left;
-	info.showsign = flags() & ios::showpos;
 	info.group = 0;
+	info.extra = 0;
+#endif
+	info.prec = prec;
+	info.width = width (0);
+	info.spec = format_char;
+	info.alt = (flags() & ios::showpoint) != 0;
+	info.left = (flags() & ios::left) != 0;
+	info.showsign = (flags() & ios::showpos) != 0;
 	info.pad = fill();
 
+#if 0
 	_IO_va_list p = (_IO_va_list) &n;
+#else
+	const void *p = (const void *) &n;
+#endif
 
 	if (__printf_fp (rdbuf(), &info, &p) < 0)
 	    set(ios::badbit|ios::failbit); // ??
