@@ -785,3 +785,15 @@ doswitch:
 	}
 	/* NOTREACHED */
 }
+
+#ifdef _IO_USE_LIBC
+int
+__vfscanf (_IO_FILE *s, const char *format, _IO_va_list argptr)
+{
+  return _IO_vfscanf (s, format, argptr, NULL);
+}
+
+#ifdef __ELF__
+#pragma weak vfscanf = __vfscanf
+#endif
+#endif
